﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Resources;
using System.Windows.Forms;
using Infragistics.Shared;
using Infragistics.Win;
using Infragistics.Win.UltraWinToolbars;
using J2.Framework.Base;
using Appearance = Infragistics.Win.Appearance;

namespace J2.Framework.Gui
{
    public class SaveToolBar<TItemList, TItem> : UltraToolbar
        where TItem : ClassBaseLoader<string>, INamedItem, new()
        where TItemList : FullLoadListBase<string, TItem>
    {
        private readonly ButtonTool m_save;
        private readonly ButtonTool m_delete;
        private readonly ComboBoxTool m_selectionCombo;

        private TItem m_oldSelection;
        private TItemList m_list;
        private string m_oldText;

        private readonly string m_itemTypeName;
        private readonly bool m_canRename;

        protected SaveToolBar(string key, string itemTypeName, bool canRename = true) : base(key)
        {
            m_itemTypeName = itemTypeName;
            m_canRename = canRename;
            ResourceManager resources = Main.ResourceManager;

            m_save = new ButtonTool("Save");
            m_save.SharedPropsInternal.Caption = "Save";
            m_save.SharedPropsInternal.Enabled = true;
            m_save.SharedPropsInternal.Category = "Save";
            m_save.SharedPropsInternal.AppearancesSmall.Appearance = new Appearance {Image = resources.GetObject("appearance1.Image")};

            m_delete = new ButtonTool("Delete");
            m_delete.SharedPropsInternal.Caption = "Delete";
            m_delete.SharedPropsInternal.Enabled = false;
            m_delete.SharedPropsInternal.Category = "Save";
            m_delete.SharedPropsInternal.AppearancesSmall.Appearance = new Appearance {Image = resources.GetObject("appearance2.Image")};

            m_selectionCombo = new ComboBoxTool("Selection") {ValueList = new ValueList(0), DropDownStyle = DropDownStyle.DropDown};
            m_selectionCombo.SharedPropsInternal.Caption = "Selection";
            m_selectionCombo.SharedPropsInternal.Category = "Save";

            m_selectionCombo.ToolValueChanged += SelectionComboOnToolValueChanged;
            m_selectionCombo.BeforeToolEnterEditMode += SelectionComboOnBeforeToolEnterEditMode;
            m_selectionCombo.AfterToolExitEditMode += SelectionComboOnAfterToolExitEditMode;

            Text = key;
        }

        public event EventHandler<SelectionChangedArgs<TItem>> SelectionChanged;

        public TItemList List
        {
            private get { return m_list; }
            set
            {
                if (m_list != null)
                {
                    m_selectionCombo.SelectedIndex = -1;
                    m_selectionCombo.ValueList.ValueListItems.Clear();

                    m_list.ListChanged -= OnListChanged;
                }

                m_list = value;

                if (m_list == null)
                {
                    return;
                }

                m_selectionCombo.ValueList.ValueListItems.AddRange(m_list.Select(item => new ValueListItem(item, item.Name)).ToArray());

                m_list.ListChanged += OnListChanged;
            }
        }

        protected TItem SelectedItem
        {
            get { return (m_selectionCombo.SelectedIndex != -1) ? (TItem) ((ValueListItem) m_selectionCombo.SelectedItem).DataValue : null; }
        }

        public bool HasUnsavedModels
        {
            get { return m_list.IsDirty || m_list.IsHierarchyDirty; }
        }

        protected virtual ToolBase[] BarTools
        {
            get { return new ToolBase[] {m_selectionCombo, m_save, m_delete}; }
        }

        protected virtual void OnSelectionChanged(SelectionChangedArgs<TItem> e)
        {
            EventHandler<SelectionChangedArgs<TItem>> handler = SelectionChanged;

            if (handler != null)
            {
                handler(this, e);
            }
        }

        protected override sealed void OnAddedToCollection(KeyedSubObjectsCollectionBase primaryCollection)
        {
            ToolBase[] tools1 = BarTools;
            ((ToolbarsCollection) primaryCollection).ToolbarsManager.Tools.AddRange(tools1);

            base.OnAddedToCollection(primaryCollection);
            Tools.AddRange(tools1);
            ToolbarsManager.ToolClick += m_toolbarsManager_ToolClick;
            ToolbarsManager.Disposed += ToolbarsManagerOnDisposed;
        }

        private void ToolbarsManagerOnDisposed(object sender, EventArgs eventArgs)
        {
            if (m_list != null)
            {
                m_list.ListChanged -= OnListChanged;
            }
        }

        private void EnableButtons()
        {
            TItem item = SelectedItem;
            bool notNullSelection = item != null;
            bool notBlank = (m_selectionCombo.SelectedIndex != -1 || m_selectionCombo.Text == "");
            EnableButtons(notNullSelection, notBlank, item);
        }

        protected virtual void EnableButtons(bool notNullSelection, bool notBlank, TItem item)
        {
            m_save.SharedProps.Enabled = notNullSelection && (item.IsDirty || SelectedItem.IsNew);
            m_delete.SharedProps.Enabled = notNullSelection && notBlank;
        }

        protected virtual void m_toolbarsManager_ToolClick(object sender, ToolClickEventArgs e)
        {
            TItem item = SelectedItem;

            if (item == null)
            {
                return;
            }

            switch (e.Tool.Key)
            {
                case "Save":
                    item.Save();
                    break;
                case "Delete":
                    if (MessageBox.Show(string.Format("Are you sure you wish to delete the {1} Model {0}", item.Name, m_itemTypeName), "Delete Model", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        List.Remove(item);
                        List.Save();
                        m_oldSelection = null;
                        m_selectionCombo.SelectedIndex = -1;
                    }
                    break;
            }
        }

        private void SelectionComboOnToolValueChanged(object sender, ToolEventArgs e)
        {
            ComboBoxTool comboBoxTool = ((ComboBoxTool) sender);

            if (!(comboBoxTool.Value is TItem))
            {
                return;
            }

            if (m_oldSelection != null)
            {
                m_oldSelection.IsDirtyChanged -= m_selectedItem_IsDirtyChanged;
            }

            m_oldSelection = null;

            TItem item = SelectedItem;

            if (item == null)
            {
                m_selectionCombo.SelectedIndex = -1;
                OnSelectionChanged(new SelectionChangedArgs<TItem>(null));
                return;
            }

            try
            {
                EnableButtons();
                OnSelectionChanged(new SelectionChangedArgs<TItem>(item));
                item.IsDirtyChanged += m_selectedItem_IsDirtyChanged;
                m_oldSelection = item;
                m_oldText = comboBoxTool.Text;
            }
            catch (Exception exception)
            {
                m_selectionCombo.AfterToolExitEditMode -= SelectionComboOnAfterToolExitEditMode;
                string msg = string.Format("Could not set {1}, {0}", exception.Message, m_itemTypeName);
                m_selectionCombo.IsInEditMode = false;
                m_selectionCombo.AfterToolExitEditMode += SelectionComboOnAfterToolExitEditMode;
                MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                UseOld();
            }
        }

        private static void SelectionComboOnBeforeToolEnterEditMode(object sender, BeforeToolEnterEditModeEventArgs beforeToolEnterEditModeEventArgs)
        {
            ComboBoxTool comboBoxTool = ((ComboBoxTool) sender);
            comboBoxTool.EditAppearance.ForeColor = Color.Red;
        }

        private void SelectionComboOnAfterToolExitEditMode(object sender, AfterToolExitEditModeEventArgs e)
        {
            ComboBoxTool comboBoxTool = ((ComboBoxTool) sender);
            comboBoxTool.EditAppearance.ForeColor = SystemColors.ControlText;

            string name = comboBoxTool.Text;

            if (m_oldSelection == null || !m_canRename)
            {
                if (m_oldSelection != null && name == m_oldSelection.Name)
                {
                    m_selectionCombo.SelectedIndex = m_oldSelection != null ? m_selectionCombo.ValueList.FindStringExact(m_oldSelection.Name) : -1;
                    return;
                }

                string text = string.Format("Do you wish to create an new {1} called {0}?", name, m_itemTypeName);

                switch (MessageBox.Show(text, string.Format("New {0}", m_itemTypeName), MessageBoxButtons.OKCancel, MessageBoxIcon.Question))
                {
                    case DialogResult.OK:
                        CreateNew(name);
                        break;
                    case DialogResult.Cancel:
                        m_selectionCombo.SelectedIndex = m_oldSelection != null ? m_selectionCombo.ValueList.FindStringExact(m_oldSelection.Name) : -1;
                        break;
                }

                return;
            }

            if (name == m_oldText)
            {
                return;
            }

            string text1 = string.Format("Do you wish to create an new {1} called {0}?\n No will rename the existing model.", name, m_itemTypeName);

            switch (MessageBox.Show(text1, string.Format("New {0}", m_itemTypeName), MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question))
            {
                case DialogResult.Yes:
                    CreateNew(name);
                    break;
                case DialogResult.No:
                    if (m_oldSelection != null)
                    {
                        m_oldSelection.Name = name;
                    }
                    break;
                case DialogResult.Cancel:
                    UseOld();
                    break;
            }
        }

        private void CreateNew(string name)
        {
            TItem item = new TItem {Name = name};
            m_list.Add(item);
            m_selectionCombo.SelectedIndex = m_selectionCombo.ValueList.FindStringExact(name);
        }

        private void UseOld()
        {
            int listIndex = m_oldSelection != null ? m_selectionCombo.ValueList.FindByDataValue(m_oldSelection).ListIndex : -1;
            m_selectionCombo.SelectedIndex = listIndex < m_selectionCombo.ValueList.ValueListItems.Count ? listIndex : -1;
        }

        private void OnListChanged(object sender, ListChangedEventArgs e)
        {
            switch (e.ListChangedType)
            {
                case ListChangedType.ItemAdded:
                    TItem item = m_list[e.NewIndex];
                    m_selectionCombo.ValueList.ValueListItems.Add(item, item.Name);
                    break;
                case ListChangedType.ItemDeleted:
                    m_selectionCombo.ValueList.ValueListItems.RemoveAt(e.NewIndex);
                    break;
            }
        }

        private void m_selectedItem_IsDirtyChanged(object sender, EventArgs e)
        {
            EnableButtons();
        }

        public void SaveList()
        {
            m_list.Save();
        }

        public override void Dispose()
        {
            if (m_list != null)
            {
                m_list.ListChanged -= OnListChanged;
            }

            base.Dispose();
        }

        public void SetSelection(string selectionName)
        {
            if (selectionName != null)
            {
                m_selectionCombo.SelectedIndex = m_selectionCombo.ValueList.FindStringExact(selectionName);
            }
        }
    }
}